@@ -9,7 +9,7 @@
 //
 /**@name interface.cpp - The interface. */
 //
-//      (c) Copyright 1998-2007 by Lutz Sammer and Jimmy Salmon
+//      (c) Copyright 1998-2008 by Lutz Sammer and Jimmy Salmon
 //
 //      This program is free software; you can redistribute it and/or modify
 //      it under the terms of the GNU General Public License as published by
@@ -25,7 +25,6 @@
 //      Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 //      02111-1307, USA.
 //
-//      $Id: interface.cpp 8922 2007-06-23 21:59:38Z jim4 $
 
 //@{
 
@@ -49,23 +48,35 @@
 #include "ui.h"
 #include "menus.h"
 #include "script.h"
-#include "tileset.h"
+#include "map.h"
 #include "minimap.h"
 #include "network.h"
 #include "font.h"
 #include "results.h"
 #include "video.h"
 #include "iolib.h"
-#include "commands.h"
+#include "replay.h"
 #include "ai.h"
 #include "widgets.h"
+#include "particle.h"
+
+/*----------------------------------------------------------------------------
+--  Defines
+----------------------------------------------------------------------------*/
+
+	/// Scrolling area (<= 15 y)
+#define SCROLL_UP     15
+	/// Scrolling area (>= VideoHeight - 16 y)
+#define SCROLL_DOWN   (Video.Height - 16)
+	/// Scrolling area (<= 15 y)
+#define SCROLL_LEFT   15
+	/// Scrolling area (>= VideoWidth - 16 x)
+#define SCROLL_RIGHT  (Video.Width - 16)
 
 /*----------------------------------------------------------------------------
 --  Variables
 ----------------------------------------------------------------------------*/
 
-static int SavedMapPositionX[3];     /// Saved map position X
-static int SavedMapPositionY[3];     /// Saved map position Y
 static char Input[80];               /// line input for messages/long commands
 static int InputIndex;               /// current index into input
 static char InputStatusLine[99];     /// Last input status line
@@ -91,7 +102,7 @@ static void ShowInput(void)
 {
 	char *input;
 
-	sprintf(InputStatusLine, _("MESSAGE:%s~!_"), Input);
+	sprintf_s(InputStatusLine, sizeof(InputStatusLine), _("MESSAGE:%s~!_"), Input);
 	input = InputStatusLine;
 	// FIXME: This is slow!
 	while (UI.StatusLine.Font->Width(input) > UI.StatusLine.Width) {
@@ -377,28 +388,6 @@ static void UiCenterOnSelected(void)
 	}
 }
 
-/**
-**  Save current map position.
-**
-**  @param position  Map position slot.
-*/
-static void UiSaveMapPosition(unsigned position)
-{
-	SavedMapPositionX[position] = UI.SelectedViewport->MapX;
-	SavedMapPositionY[position] = UI.SelectedViewport->MapY;
-}
-
-/**
-**  Recall map position.
-**
-**  @param position  Map position slot.
-*/
-static void UiRecallMapPosition(unsigned position)
-{
-	UI.SelectedViewport->Set(
-		SavedMapPositionX[position], SavedMapPositionY[position], TileSizeX / 2, TileSizeY / 2);
-}
-
 /**
 **  Toggle terrain display on/off.
 */
@@ -418,12 +407,9 @@ static void UiToggleTerrain(void)
 static void UiFindIdleWorker(void)
 {
 	CUnit *unit;
-	// FIXME: static variable, is not needed.
-	static CUnit *LastIdleWorker = NoUnitP;
 
-	unit = FindIdleWorker(ThisPlayer, LastIdleWorker);
+	unit = FindIdleWorker(ThisPlayer);
 	if (unit != NoUnitP) {
-		LastIdleWorker = unit;
 		SelectSingleUnit(unit);
 		UI.StatusLine.Clear();
 		ClearCosts();
@@ -545,9 +531,9 @@ static bool CommandKey(int key)
 		case SDLK_F3:
 		case SDLK_F4: // Set/Goto place
 			if (KeyModifiers & ModifierShift) {
-				UiSaveMapPosition(key - SDLK_F2);
+				SetSavedMapPosition(key - SDLK_F2, UI.SelectedViewport->MapX, UI.SelectedViewport->MapY);
 			} else {
-				UiRecallMapPosition(key - SDLK_F2);
+				RecallSavedMapPosition(key - SDLK_F2);
 			}
 			break;
 
@@ -682,14 +668,21 @@ static bool CommandKey(int key)
 */
 int HandleCheats(const std::string &input)
 {
-	int ret;
+	int ret = 0;
 
 #ifdef DEBUG
 	if (input == "ai me") {
 		if (ThisPlayer->AiEnabled) {
+			// FIXME: UnitGoesUnderFog and UnitGoesOutOfFog change unit refs
+			// for human players.  We can't switch back to a human player or
+			// we'll be using the wrong ref counts.
+#if 0
 			ThisPlayer->AiEnabled = 0;
 			ThisPlayer->Type = PlayerPerson;
 			SetMessage("AI is off, Normal Player");
+#else
+			SetMessage("Cannot disable 'ai me' cheat");
+#endif
 		} else {
 			ThisPlayer->AiEnabled = 1;
 			ThisPlayer->Type = PlayerComputer;
@@ -706,17 +699,18 @@ int HandleCheats(const std::string &input)
 	lua_gettable(Lua, LUA_GLOBALSINDEX);
 	if (!lua_isfunction(Lua, -1)) {
 		DebugPrint("No HandleCheats function in lua.\n");
-		return 0;
-	}
-	lua_pushstring(Lua, input.c_str());
-	LuaCall(1, 0);
-	if (lua_gettop(Lua) - base == 1) {
-		ret = LuaToBoolean(Lua, -1);
-		lua_pop(Lua, 1);
 	} else {
-		LuaError(Lua, "HandleCheats must return a boolean");
-		ret = 0;
+		lua_pushstring(Lua, input.c_str());
+		if (LuaCall(1, 0, false)) {
+			// A Lua error occurred within HandleCheats,
+			// and LuaCall already reported it.
+		} else if (lua_gettop(Lua) - base != 1) {
+			DebugPrint("HandleCheats must return a boolean\n");
+		} else {
+			ret = LuaToBoolean(Lua, -1);
+		}
 	}
+	lua_settop(Lua, base);
 	return ret;
 }
 
@@ -729,7 +723,6 @@ int HandleCheats(const std::string &input)
 static int InputKey(int key)
 {
 	char ChatMessage[sizeof(Input) + 40];
-	int i;
 	char *namestart;
 	char *p;
 	char *q;
@@ -749,7 +742,7 @@ static int InputKey(int key)
 			if (Input[0] == '-') {
 				if (!GameObserve && !GamePaused) {
 					CommandLog("input", NoUnitP, FlushCommands, -1, -1, NoUnitP, Input, -1);
-					CclCommand(Input + 1);
+					CclCommand(Input + 1, false);
 				}
 			} else
 #endif
@@ -783,7 +776,7 @@ static int InputKey(int key)
 						++p;
 					}
 				}
-				sprintf(ChatMessage, "~%s~<%s>~> %s",
+				sprintf_s(ChatMessage, sizeof(ChatMessage), "~%s~<%s>~> %s",
 					PlayerColorNames[ThisPlayer->Index].c_str(),
 					ThisPlayer->Name.c_str(), Input);
 				// FIXME: only to selected players ...
@@ -818,7 +811,7 @@ static int InputKey(int key)
 			if (!strlen(namestart)) {
 				return 1;
 			}
-			for (i = 0; i < PlayerMax; ++i) {
+			for (int i = 0; i < PlayerMax; ++i) {
 				if (Players[i].Type != PlayerPerson) {
 					continue;
 				}
@@ -870,7 +863,7 @@ static void Screenshot(void)
 
 	for (i = 1; i <= 99; ++i) {
 		// FIXME: what if we can't write to this directory?
-		sprintf(filename, "screen%02d.png", i);
+		sprintf_s(filename, sizeof(filename), "screen%02d.png", i);
 		if (fd.open(filename, CL_OPEN_READ) == -1) {
 			break;
 		}
